##
## TOOL DESCRIPTION
##
## Based on Rick Wilson Xline script
## 
## Transforms edges into construction lines; transforms constructions lines into edges
## Activates from contextual menu or toolbar
## Alternative mode (command) Changes construction lines stipples and edge color
## ALT (Macintosh) allows to add endpoints
## COMMAND (Macintosh) toggles between the two modes.
## Objects can be selected prior activation and will be changed, either by context
## the menu, or by clicking the tool. Then the tool remains active until another
## tool is selected.
## Right click in empty space to set the stipple and the color.
## Color and stipple choices remains throughout session, but returns to black and default next stime you start SU
##
##
## Tool copyright Edge LLC Michael Vulpillat
## Edgecons@usa.net
## Based on an original tool developped by
## Rick Wilson

## Version history:
## Mar 2012 V 1.2 better integration, simplification
## Mar 2012 V 1.1 Added color change
## Mar 2012 Original version 1.0 (Bcurrent)


# Permission to use, copy, modify, and distribute this software without fee is 
# hereby granted for non commercial purpose only and, provided that the above
# copyright notice appear in all copies.
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# When used or intended to use for commercial purpose, i.e. Monetary benefit
# is received in connection with the use of this software, please contact 
# the author at EDGE LLC, edgecons@usa.net for licensing terms and conditions.require 'sketchup.rb'

@@colorx = "Black"
@@stipplex = "_"

class XLinemTool

def reset
	@endpoints = false
	@change = false
	@colorlist = String.new
	colorlist = Sketchup::Color.names
	l = colorlist.size
	l = l-1
	for e in 0..l do
	 @colorlist = @colorlist + colorlist[e].to_s + "|" if e != l
	 @colorlist = @colorlist + colorlist[e].to_s if e == l
	end
	cflag = false
end

def activate
	@model=Sketchup.active_model
	@ents=@model.active_entities
	@sel=@model.selection
	@lines=[]
	reset
	cursor
	show_status
	conversion
end

def show_status
		statustext = "Edges into \"" + @@stipplex + "\" construction lines and cline to " + @@colorx + " edges (Command)" if !@change
		statustext = "Construction lines into \"" + @@stipplex + "\" clines and edges into " + @@colorx + " edges (Command)" if @change
		statusoption = "; Draw end points (Alt)" if @endpoints
		statusoption = "; Endpoints not drawn (Alt)" if !@endpoints
		status_label = statustext + statusoption
    Sketchup::set_status_text(status_label,SB_PROMPT)
end

def cursor  # determination of proper cursor to use
	@cursor = nil  # required for it to function properly
  curpath = File.dirname(__FILE__)+"/Xlinem/Xline_SM.png"
  @cursor = UI::create_cursor(curpath,0,24)  if curpath
end

def onSetCursor  # standard method to set custom cursor
	UI::set_cursor (@cursor)
end	
	
def conversion
	@model.start_operation "Line Conversion"
	view = @model.active_view
	if !@change
		for e in @sel
			if e.class == Sketchup::Edge
				lstart = e.start.position
				lend = e.end.position
				if @endpoints
					@ents.add_cpoint lstart
					@ents.add_cpoint lend
#				elsif !@endpoints
#					cent = e.entities
#					for en in cent
#						if en == Sketchup::ConstructionPoint
#							@ents.erase_entities en
#						end
#					end
				end
				@ents.add_cline(lstart,lend, @@stipplex) 
				@lines.push(e)
			elsif e.class == Sketchup::ConstructionLine
				if e.start and e.end
					lstart = e.start
					lend = e.end
						if @endpoints
							@ents.add_cpoint lstart
							@ents.add_cpoint lend
						end
					@ents.add_edges(lstart,lend)
					@ents[-1].material= @@colorx.to_s
					@ents [-1].find_faces
#					if !@endpoints
#						cent = @ents [-1]
#						cent = cent.entities
#						for en in cent
#							if en == Sketchup::ConstructionPoint
#								@ents.erase_entities en
#							end
#						end
#					end
					@lines.push(e)
				end
			end # e.class
		end # for e
		if not @lines.empty?
			for e in @lines
				e.erase! if e.valid? #if e.faces.empty?
			end
		end
	elsif @change
		for e in @sel
			if e.class == Sketchup::ConstructionLine
				e.stipple= @@stipplex
				if (e.start and e.end)
					lstart = e.start
					lend = e.end
						if @endpoints
							@ents.add_cpoint lstart
							@ents.add_cpoint lend
						end
				end
			elsif e.class == Sketchup::Edge
				lstart = e.start.position
				lend = e.end.position
				if @endpoints
					@ents.add_cpoint lstart
					@ents.add_cpoint lend
				end
				e.material= @@colorx.to_s
			end	# e.class	
			view.invalidate
		end # for e

=begin				lstart = e.start
				lend = e.end
				e.erase!
				view.drawing_color= @color
				@ents.add_cline (lstart,lend)
			elsif e.class == Sketchup::Edge
				lstart = e.start.position
				lend = e.end.position
				e.erase!
				view.drawing_color= @color
				@ents.add_edges (lstart,lend)
				@ents [-1].find_faces
=end

	end # @change	 
	@sel.clear
	@model.commit_operation
end # conversion



def self.xLine_validate_selection_edge
	Sketchup.active_model.selection.each {|e| return true if e.class == Sketchup::Edge}
	return nil
end

def self.xLine_validate_selection_xline
	Sketchup.active_model.selection.each {|e| return true if (e.class == Sketchup::ConstructionLine and e.start and e.end)}
	return nil
end


def onKeyDown(key, repeat, flags, view)
    if key == VK_COMMAND  ### Cmd toggles stipples and change
			@change = !@change
=begin			@stip += 1
			case @stip
			when 1 
				@stipple = "-"
			when 2 
				@stipple = "."
			when 3 
				@stipple = "-.-"
			when 4 
				@stipple = "_"
			when 6
				@stipple = "_"
				@stip = 0
			end # case
=end
		end #if

        ### 
    if key == VK_ALT ### Alt toggles end points
			@endpoints = !@endpoints
		end#if
		show_status
    ###
end

def onLButtonDown flags, x, y, view 
	ph = view.pick_helper
	ph.do_pick x,y
	best = ph.best_picked
	@sel.add best if best
	conversion
end

def onRButtonDown flags, x, y, view 
		cflag = true if @@colorx == "Black"
		list = ["-|.|-.-|_",@colorlist]
		prompt = ["XLine Stipple: ", "Color: "]
		values = [@@stipplex, @@colorx]
		result = inputbox(prompt,values,list,"Change Parameters")
		if result
			@@stipplex = result[0].to_s
			@@colorx = result[1].to_s
			if cflag and @model.rendering_options["EdgeColorMode"] != 0
				choice = UI.messagebox "The edges color will only be visible if you switch to \"display color by material\". Do you want to do this at this time?", MB_YESNOCANCEL
				case choice
				when 6
					@model.rendering_options["EdgeColorMode"]= 0 # sets up styles for Color By material
					cflag = false
				when 7
					cflag = false
				end
			end # if cflag
		end # if result
		show_status
		conversion
end # def

def deactivate(view)
		reset
    Sketchup::set_status_text("",SB_PROMPT)
    Sketchup::set_status_text("",SB_VCB_LABEL)
    Sketchup::set_status_text("",SB_VCB_VALUE)
end


end  #class XLinemTool


if( not file_loaded?("XLinem.rb") )
	UI.add_context_menu_handler do |menu|
		if XLinemTool.xLine_validate_selection_edge
			menu.add_separator
			menu.add_item("Convert to XLine") { Sketchup.active_model.select_tool XLinemTool.new }
		elsif XLinemTool.xLine_validate_selection_xline
			menu.add_separator
			menu.add_item("Convert to Edge") { Sketchup.active_model.select_tool XLinemTool.new }		
		end
	end
end

#-----------------------------------------------------------------------------



unless file_loaded?(__FILE__)

    cmd = UI::Command.new("xlinem Convert")  {Sketchup.active_model.select_tool XLinemTool.new }
    cmd.small_icon = File.join(File.dirname(__FILE__), "Xlinem", "Xline_SM.png")
    cmd.large_icon = File.join(File.dirname(__FILE__), "Xlinem", "Xline_LG.png")
    cmd.tooltip = "Construction Line Conversion"
    tb = UI.toolbar("Line Conversion")
    tb.add_item(cmd)
    tb.restore
		
end
file_loaded(__FILE__)
